@@ -1,5 +1,3 @@
-from __future__ import unicode_literals
-
 from django import forms
 from django.contrib import admin
 from django.contrib.auth import get_user_model
@@ -27,15 +25,8 @@ from .subjects import (
 class EventAdmin(SubjectBaseAdmin):
     subject_type = SubjectType.EVENT
     registration_model = EventRegistration
-    list_display    = (
-        'id', 'name', 'subject_type', 'get_groups_list', 'get_leaders_list',
-        'get_times_list',
-        'place', 'public', 'registration_allowed_icon',
-        'get_registrations_link',
-        'icon', 'note',
-    )
     list_export     = (
-        'id', 'name', 'department', 'subject_type', 'get_groups_list', 'get_leaders_list',
+        'id', 'code', 'name', 'department', 'subject_type', 'get_groups_list', 'get_leaders_list',
         'start_date', 'start_time', 'end_date', 'end_time',
         'place', 'public',
         'get_approved_registrations_count', 'get_unapproved_registrations_count', 'note',
@@ -104,7 +95,7 @@ class EventAdmin(SubjectBaseAdmin):
 class EventRegistrationAdmin(PdfExportAdminMixin, SubjectRegistrationBaseAdmin):
     list_display    = (
         'variable_symbol', 'download_tag', 'subject_name', 'participant', 'price', 'event_discounts', 'event_payments',
-        'created', 'approved', 'payment_requested', 'cancel_request', 'canceled',
+        'created', 'approved', 'payment_requested', 'cancel_request', 'canceled', 'note',
     )
     list_filter     = (
         ('subject__school_year',    SchoolYearListFilter),
@@ -119,7 +110,7 @@ class EventRegistrationAdmin(PdfExportAdminMixin, SubjectRegistrationBaseAdmin):
     def event_discounts(self, obj):
         status = obj.get_payment_status()
         return format_html(
-            '<a class="popup-link" href="{href_list}"><b>{amount}</b></a>'
+            '<a href="{href_list}"><b>{amount}</b></a>'
             ' &nbsp; <a class="popup-link" href="{href_add}" style="background-position: 0 0" title="{title_add}">'
             '<img src="{icon_add}" alt="+"/></a>',
             href_list   = reverse('admin:leprikon_eventdiscount_changelist') + '?registration={}'.format(obj.id),
@@ -151,8 +142,5 @@ class EventRegistrationAdmin(PdfExportAdminMixin, SubjectRegistrationBaseAdmin):
 
 
 class EventDiscountAdmin(PdfExportAdminMixin, SubjectPaymentBaseAdmin):
-    list_display    = ('created', 'registration', 'subject', 'amount_html', 'explanation')
-    list_export     = ('created', 'registration', 'subject', 'amount', 'explanation')
-
-    def get_model_perms(self, request):
-        return {}
+    list_display    = ('accounted', 'registration', 'subject', 'amount_html', 'explanation')
+    list_export     = ('accounted', 'registration', 'subject', 'amount', 'explanation')
