@@ -1,10 +1,9 @@
-from __future__ import unicode_literals
-
 from django.contrib import admin
 
 
 class LeprikonSiteAdmin(admin.ModelAdmin):
     list_display    = ('name',)
+    filter_horizontal = ('registration_agreements',)
 
     def has_add_permission(self, request):
         return False
@@ -17,3 +16,9 @@ class LeprikonSiteAdmin(admin.ModelAdmin):
         if 'delete_selected' in actions:
             del(actions['delete_selected'])
         return actions
+
+    def get_fields(self, request, obj=None):
+        fields = super(LeprikonSiteAdmin, self).get_fields(request, obj)
+        if not obj or not obj.old_registration_agreement:
+            fields.remove('old_registration_agreement')
+        return fields
