@@ -1,19 +1,23 @@
-from __future__ import unicode_literals
-
 from json import dumps
 
+from bankreader.models import Transaction as BankreaderTransaction
 from django import forms
+from django.conf.urls import url as urls_url
 from django.contrib import admin
 from django.contrib.admin.templatetags.admin_list import _boolean_icon
 from django.contrib.auth import get_user_model
+from django.contrib.auth.decorators import permission_required
+from django.contrib.staticfiles.templatetags.staticfiles import static
 from django.core.urlresolvers import reverse
-from django.shortcuts import render
+from django.http import HttpResponseBadRequest, JsonResponse
+from django.shortcuts import get_object_or_404, render
 from django.utils.html import format_html
 from django.utils.translation import ugettext_lazy as _
 
 from ..forms.subjects import RegistrationAdminForm
 from ..models.subjects import (
-    Subject, SubjectAttachment, SubjectTypeAttachment, SubjectVariant,
+    Subject, SubjectAttachment, SubjectPayment, SubjectTypeAttachment,
+    SubjectVariant,
 )
 from ..utils import amount_color, currency
 from .export import AdminExportMixin
@@ -36,12 +40,18 @@ class SubjectTypeAdmin(admin.ModelAdmin):
     list_display    = ('plural', 'order')
     list_editable   = ('order',)
     exclude         = ('order',)
-    filter_horizontal = ('questions',)
+    filter_horizontal = ('questions', 'registration_agreements')
     prepopulated_fields = {'slug': ('plural',)}
     inlines         = (
         SubjectTypeAttachmentInlineAdmin,
     )
 
+    def get_fields(self, request, obj=None):
+        fields = super(SubjectTypeAdmin, self).get_fields(request, obj)
+        if not obj or not obj.old_registration_agreement:
+            fields.remove('old_registration_agreement')
+        return fields
+
 
 
 class SubjectGroupAdmin(admin.ModelAdmin):
@@ -66,6 +76,13 @@ class SubjectVariantInlineAdmin(admin.TabularInline):
 class SubjectBaseAdmin(AdminExportMixin, SendMessageAdminMixin, admin.ModelAdmin):
     subject_type    = None
     registration_model = None
+    list_display    = (
+        'id', 'code', 'name', 'subject_type', 'get_groups_list', 'get_leaders_list',
+        'get_times_list',
+        'place', 'public', 'registration_allowed_icon',
+        'get_registrations_link',
+        'get_journal_link', 'icon', 'note',
+    )
     list_editable   = ('public', 'note')
     list_filter     = (
         ('school_year',     SchoolYearListFilter),
@@ -78,11 +95,23 @@ class SubjectBaseAdmin(AdminExportMixin, SendMessageAdminMixin, admin.ModelAdmin
         SubjectVariantInlineAdmin,
         SubjectAttachmentInlineAdmin,
     )
-    filter_horizontal = ('age_groups', 'groups', 'leaders', 'questions')
+    filter_horizontal = ('age_groups', 'groups', 'leaders', 'questions', 'registration_agreements')
     actions         = ('set_registration_dates',)
     search_fields   = ('name', 'description')
     save_as         = True
 
+    @property
+    def media(self):
+        m = super(SubjectBaseAdmin, self).media
+        m.add_js(['leprikon/js/Popup.js'])
+        return m
+
+    def get_fields(self, request, obj=None):
+        fields = super(SubjectBaseAdmin, self).get_fields(request, obj)
+        if not obj or not obj.old_registration_agreement:
+            fields.remove('old_registration_agreement')
+        return fields
+
     def get_form(self, request, obj=None, **kwargs):
         form = super(SubjectBaseAdmin, self).get_form(request, obj, **kwargs)
         if obj:
@@ -105,6 +134,15 @@ class SubjectBaseAdmin(AdminExportMixin, SendMessageAdminMixin, admin.ModelAdmin
         obj.school_year = request.school_year
         return obj
 
+    def get_journal_link(self, obj):
+        return '<a href="{url}" title="{title}" target="_blank">{journal}</a>'.format(
+            url     = reverse('admin:leprikon_subject_journal', args=[obj.id]),
+            title   = _('printable journal'),
+            journal = _('journal'),
+        )
+    get_journal_link.short_description = _('journal')
+    get_journal_link.allow_tags = True
+
     def get_message_recipients(self, request, queryset):
         return get_user_model().objects.filter(
             leprikon_subjectregistrations__subject__in = queryset
@@ -124,7 +162,8 @@ class SubjectBaseAdmin(AdminExportMixin, SendMessageAdminMixin, admin.ModelAdmin
         else:
             icon = True
             title = ''
-        return '<a href="{url}" title="{title}">{icon} {approved}{unapproved}</a>'.format(
+        return format_html(
+            '<a href="{url}" title="{title}">{icon} {approved}{unapproved}</a>',
             url     = reverse('admin:{}_{}_changelist'.format(
                 self.registration_model._meta.app_label,
                 self.registration_model._meta.model_name,
@@ -132,7 +171,16 @@ class SubjectBaseAdmin(AdminExportMixin, SendMessageAdminMixin, admin.ModelAdmin
             title       = title,
             icon        = _boolean_icon(icon),
             approved    = approved_registrations_count,
-            unapproved = ' + {}'.format(unapproved_registrations_count) if unapproved_registrations_count else '',
+            unapproved  = ' + {}'.format(unapproved_registrations_count) if unapproved_registrations_count else '',
+        ) + format_html(
+            '<a class="popup-link" href="{url}" style="background-position: 0 0" title="{title}">'
+            '<img src="{icon}" alt="+"/></a>',
+            url     = reverse('admin:{}_{}_add'.format(
+                self.registration_model._meta.app_label,
+                self.registration_model._meta.model_name,
+            )) + '?subject={}'.format(obj.id),
+            title   = _('add registration'),
+            icon    = static('admin/img/icon-addlink.svg'),
         )
     get_registrations_link.short_description = _('registrations')
     get_registrations_link.allow_tags = True
@@ -190,11 +238,12 @@ class SubjectBaseAdmin(AdminExportMixin, SendMessageAdminMixin, admin.ModelAdmin
 class SubjectAdmin(AdminExportMixin, SendMessageAdminMixin, admin.ModelAdmin):
     """ Hidden admin used for raw id fields """
     list_display    = (
-        'id', 'name', 'subject_type', 'get_groups_list', 'get_leaders_list', 'icon',
+        'id', 'code', 'name', 'subject_type', 'get_groups_list', 'get_leaders_list', 'icon',
     )
     list_filter     = (
         ('school_year',     SchoolYearListFilter),
         'department',
+        'subject_type__subject_type',
         ('subject_type',    SubjectTypeListFilter),
         ('groups',          SubjectGroupListFilter),
         ('leaders',         LeaderListFilter),
@@ -215,9 +264,23 @@ class SubjectAdmin(AdminExportMixin, SendMessageAdminMixin, admin.ModelAdmin):
     icon.allow_tags = True
     icon.short_description = _('photo')
 
+    def get_urls(self):
+        urls = super(SubjectAdmin, self).get_urls()
+        return [urls_url(
+            r'(?P<subject_id>\d+)/journal/$',
+            self.admin_site.admin_view(self.journal),
+            name='leprikon_subject_journal',
+        )] + urls
+
+    def journal(self, request, subject_id):
+        return render(request, 'leprikon/subject_journal.html', {
+            'subject': get_object_or_404(Subject, id=subject_id),
+            'admin': True,
+        })
 
 
 class SubjectRegistrationBaseAdmin(AdminExportMixin, SendMessageAdminMixin, admin.ModelAdmin):
+    list_editable   = ('note',)
     list_export     = (
         'id', 'variable_symbol', 'slug', 'created', 'payment_requested', 'approved', 'user', 'subject', 'price',
         'answers', 'cancel_request', 'canceled',
@@ -228,7 +291,7 @@ class SubjectRegistrationBaseAdmin(AdminExportMixin, SendMessageAdminMixin, admi
         'has_parent1', 'parent1_first_name', 'parent1_last_name', 'parent1_street', 'parent1_city',
         'parent1_postal_code', 'parent1_phone', 'parent1_email',
         'has_parent2', 'parent2_first_name', 'parent2_last_name', 'parent2_street', 'parent2_city',
-        'parent2_postal_code', 'parent2_phone', 'parent2_email',
+        'parent2_postal_code', 'parent2_phone', 'parent2_email', 'note',
     )
     list_filter     = (
         ('subject__school_year',    SchoolYearListFilter),
@@ -255,9 +318,6 @@ class SubjectRegistrationBaseAdmin(AdminExportMixin, SendMessageAdminMixin, admi
         m.add_js(['leprikon/js/Popup.js'])
         return m
 
-    def has_add_permission(self, request):
-        return False
-
     def has_delete_permission(self, request, obj=None):
         return obj and obj.approved is None and obj.payments.count() == 0
 
@@ -285,22 +345,57 @@ class SubjectRegistrationBaseAdmin(AdminExportMixin, SendMessageAdminMixin, admi
         self.message_user(request, _('Selected registrations were canceled.'))
     cancel.short_description = _('Cancel selected registrations')
 
+    def formfield_for_foreignkey(self, db_field, request, **kwargs):
+        formfield = super(SubjectRegistrationBaseAdmin, self).formfield_for_foreignkey(db_field, request, **kwargs)
+        if db_field.name == 'subject':
+            limit_choices_to = {'subject_type__subject_type__exact': self.model.subject_type}
+            formfield.limit_choices_to = limit_choices_to
+            formfield.widget.rel.limit_choices_to = limit_choices_to
+        return formfield
+
     def get_form(self, request, obj, **kwargs):
-        questions       = obj.subject.all_questions
-        answers         = obj.get_answers()
-        kwargs['form']  = type(RegistrationAdminForm.__name__, (RegistrationAdminForm,), dict(
-            ('q_' + q.name, q.get_field(initial=answers.get(q.name, None)))
-            for q in questions
-        ))
-        request._leprikon_registration = obj
+        try:
+            # first try request.POST (user may want to change the subject)
+            subject = Subject.objects.get(id=int(request.POST.get('subject')))
+        except (Subject.DoesNotExist, TypeError, ValueError):
+            if obj:
+                # use subject from object
+                subject = obj.subject
+            else:
+                # try to get subject from request.GET
+                try:
+                    subject = Subject.objects.get(id=int(request.GET.get('subject')))
+                except (Subject.DoesNotExist, TypeError, ValueError):
+                    subject = None
+        if subject:
+            questions       = subject.all_questions
+            answers         = obj.get_answers() if obj else {}
+            kwargs['form']  = type(RegistrationAdminForm.__name__, (RegistrationAdminForm,), dict(
+                ('q_' + q.name, q.get_field(initial=answers.get(q.name)))
+                for q in questions
+            ))
+        else:
+            kwargs['fields'] = ['subject']
         return super(SubjectRegistrationBaseAdmin, self).get_form(request, obj, **kwargs)
 
+    def get_inline_instances(self, request, obj=None):
+        return super(SubjectRegistrationBaseAdmin, self).get_inline_instances(request, obj) if obj else []
+
     def save_form(self, request, form, change):
-        questions   = form.instance.subject.all_questions
-        answers     = {}
-        for q in questions:
-            answers[q.name] = form.cleaned_data['q_' + q.name]
-        form.instance.answers = dumps(answers)
+        # only override this method for change form (not changelist form)
+        if not form.prefix:
+            # set price
+            if not form.instance.id:
+                form.instance.price = (
+                    form.instance.subject_variant.price
+                    if form.instance.subject_variant
+                    else form.instance.subject.price
+                )
+            questions   = form.instance.subject.all_questions
+            answers     = {}
+            for q in questions:
+                answers[q.name] = form.cleaned_data['q_' + q.name]
+            form.instance.answers = dumps(answers)
         return super(SubjectRegistrationBaseAdmin, self).save_form(request, form, change)
 
     def subject_name(self, obj):
@@ -329,6 +424,12 @@ class SubjectRegistrationAdmin(AdminExportMixin, SendMessageAdminMixin, admin.Mo
         ('subject__leaders',        LeaderListFilter),
     )
     ordering        = ('-created',)
+    search_fields   = (
+        'variable_symbol', 'participant_birth_num',
+        'participant_first_name', 'participant_last_name',
+        'parent1_first_name', 'parent1_last_name',
+        'parent2_first_name', 'parent2_last_name',
+    )
     fieldsets       = ((None, {'fields': ()}),)
 
     def get_model_perms(self, request):
@@ -356,15 +457,32 @@ class SubjectPaymentBaseAdmin(AdminExportMixin, admin.ModelAdmin):
         'registration__subject__name', 'registration__participant_first_name', 'registration__participant_last_name',
         'registration__participant_birth_num',
     )
-    date_hierarchy  = 'created'
-    ordering        = ('-created',)
+    date_hierarchy  = 'accounted'
+    ordering        = ('-accounted',)
     raw_id_fields   = ('registration',)
+    closed_fields   = ('accounted', 'registration', 'amount')
+
+    def is_closed(self, request, obj):
+        return (
+            obj and request.leprikon_site.max_closure_date and
+            request.leprikon_site.max_closure_date > obj.accounted.date()
+        )
 
     def has_delete_permission(self, request, obj=None):
-        return False
+        if self.is_closed(request, obj):
+            return False
+        else:
+            return super(SubjectPaymentBaseAdmin, self).has_delete_permission(request, obj)
 
     def get_readonly_fields(self, request, obj=None):
-        return obj and ('registration', 'amount') or ()
+        if obj:
+            # it is strange, but obj given to this method contains values from request.POST
+            # but we need to decide according to current state in database
+            obj = self.model.objects.get(pk=obj.pk)
+        if self.is_closed(request, obj):
+            return self.closed_fields
+        else:
+            return ()
 
     def get_actions(self, request):
         actions = super(SubjectPaymentBaseAdmin, self).get_actions(request)
@@ -388,13 +506,57 @@ class SubjectPaymentBaseAdmin(AdminExportMixin, admin.ModelAdmin):
 
 
 class SubjectPaymentAdmin(PdfExportAdminMixin, SubjectPaymentBaseAdmin):
-    list_display    = ('created', 'download_tag', 'registration', 'subject', 'payment_type_label', 'amount_html',
+    list_display    = ('accounted', 'download_tag', 'registration', 'subject', 'payment_type_label', 'amount_html',
                        'received_by', 'note')
     list_editable   = ('note',)
-    list_export     = ('created', 'registration', 'subject', 'payment_type_label', 'amount')
-    raw_id_fields   = ('registration', 'related_payment',)
+    list_export     = ('accounted', 'registration', 'subject', 'payment_type_label', 'amount')
+    raw_id_fields   = ('registration', 'related_payment', 'bankreader_transaction', 'pays_payment')
     exclude         = ('received_by',)
 
+    def get_urls(self):
+        urls = super(SubjectPaymentAdmin, self).get_urls()
+        populate_view = self.admin_site.admin_view(permission_required('leprikon.add_subjectpayment')(self.populate))
+        return [
+            urls_url(r'populate.json$', populate_view, name='leprikon_subjectpayment_populate')
+        ] + urls
+
+    def populate(self, request):
+        if 'related_payment' in request.GET:
+            try:
+                related_payment = get_object_or_404(
+                    SubjectPayment,
+                    id=int(request.GET['related_payment']),
+                    payment_type__in=(SubjectPayment.PAYMENT_TRANSFER, SubjectPayment.RETURN_TRANSFER),
+                )
+            except ValueError:
+                return HttpResponseBadRequest()
+            return JsonResponse({
+                'amount': - related_payment.amount,
+                'payment_type': (
+                    SubjectPayment.RETURN_TRANSFER
+                    if related_payment.payment_type == SubjectPayment.PAYMENT_TRANSFER
+                    else SubjectPayment.PAYMENT_TRANSFER
+                )
+            })
+        elif 'bankreader_transaction' in request.GET:
+            try:
+                bankreader_transaction = get_object_or_404(
+                    BankreaderTransaction,
+                    id=int(request.GET['bankreader_transaction']),
+                )
+            except ValueError:
+                return HttpResponseBadRequest()
+            return JsonResponse({
+                'amount': bankreader_transaction.amount,
+                'payment_type': (
+                    SubjectPayment.PAYMENT_BANK
+                    if bankreader_transaction.amount > 0
+                    else SubjectPayment.RETURN_BANK
+                )
+            })
+        else:
+            return HttpResponseBadRequest()
+
     def save_model(self, request, obj, form, change):
         if not change:
             obj.received_by = request.user
