@@ -1,5 +1,3 @@
-from __future__ import unicode_literals
-
 from django.conf.urls import url
 
 from . import views
@@ -10,5 +8,5 @@ def l_url(pattern, name):
 
 
 urlpatterns = [
-    l_url(r'^course-registrations/(?P<course_id>[0-9]+)/$',     'course_registrations'),
+    l_url(r'^registrations/(?P<subject_id>[0-9]+)/$', 'registrations'),
 ]
