@@ -1,9 +1,8 @@
-# -*- coding: utf-8 -*-
-from __future__ import unicode_literals
-
 from django.apps import AppConfig
 from django.utils.translation import ugettext_lazy as _
 
+from . import bankreaders  # noqa
+
 
 class LeprikonConfig(AppConfig):
     name            = 'leprikon'
@@ -35,7 +34,7 @@ class LeprikonConfig(AppConfig):
                 in_navigation=True,
                 navigation_extenders='LeprikonMenu',
                 published=True,
-            )
+            ).set_as_homepage()
             menu_pool.clear()
         except:
             pass
