@@ -1,11 +1,16 @@
-from __future__ import unicode_literals
-
+from datetime import date
 from json import dumps
 
 from django import forms
+from django.contrib.auth import get_user_model
+from django.db import transaction
 from django.db.models import Q
+from django.db.utils import IntegrityError
+from django.utils.crypto import get_random_string
+from django.utils.functional import cached_property
 from django.utils.timezone import now
 from django.utils.translation import ugettext_lazy as _
+from verified_email_field.forms import VerifiedEmailField
 
 from ..models.agegroup import AgeGroup
 from ..models.courses import Course, CourseRegistration
@@ -14,11 +19,14 @@ from ..models.events import Event, EventRegistration
 from ..models.fields import DAY_OF_WEEK
 from ..models.place import Place
 from ..models.roles import Leader, Parent, Participant
+from ..models.school import School
 from ..models.subjects import Subject, SubjectGroup, SubjectType
 from ..utils import get_age, get_birth_date
 from .form import FormMixin
 from .widgets import RadioSelectBootstrap
 
+User = get_user_model()
+
 
 class SubjectFilterForm(FormMixin, forms.Form):
     q               = forms.CharField(label=_('Search term'), required=False)
@@ -139,15 +147,21 @@ class SubjectForm(FormMixin, forms.ModelForm):
 
 
 class RegistrationForm(FormMixin, forms.ModelForm):
+    participant_school = forms.ChoiceField(
+        label=_('School'),
+        choices=lambda: [('', '---------')] + [
+            (school.id, school) for school in School.objects.all()
+        ] + [('other', _('other school'))]
+    )
 
     def __init__(self, subject, user, **kwargs):
         super(RegistrationForm, self).__init__(**kwargs)
+        self.user = user
         self.instance.subject = subject
-        self.instance.user = user
         self.participants = user.leprikon_participants.exclude(
             birth_num__in=self.instance.subject.active_registrations.values_list('participant_birth_num', flat=True)
-        )
-        self.parents = user.leprikon_parents.all()
+        ) if user.is_authenticated() else []
+        self.parents = user.leprikon_parents.all() if user.is_authenticated() else []
 
         # choices for subject_variant
         if self.instance.subject.all_variants:
@@ -180,6 +194,10 @@ class RegistrationForm(FormMixin, forms.ModelForm):
 
         del kwargs['instance']
 
+        if not user.is_authenticated():
+            kwargs['prefix'] = 'email'
+            self.email_form = self.EmailForm(**kwargs)
+
         class ParticipantSelectForm(FormMixin, forms.Form):
             participant = forms.ChoiceField(
                 label   = _('Choose'),
@@ -207,20 +225,114 @@ class RegistrationForm(FormMixin, forms.ModelForm):
             (q.name, q.get_field()) for q in self.instance.subject.all_questions
         ))(**kwargs)
 
-        kwargs['prefix'] = 'agreement'
-        self.agreement_form = self.AgreementForm(**kwargs)
+        self.agreement_forms = []
+        for agreement in self.instance.subject.all_registration_agreements:
+            kwargs['prefix'] = 'agreement_%s' % agreement.id
+            form_attributes = {
+                'agreement': agreement,
+                'options': {},
+            }
+            for option in agreement.all_options:
+                option_id = 'option_%s' % option.id
+                form_attributes[option_id] = forms.BooleanField(
+                    label=option.option,
+                    required=option.required,
+                )
+                form_attributes['options'][option_id] = option
+            AgreementForm = type(str('AgreementForm'), (FormMixin, forms.Form), form_attributes)
+            self.agreement_forms.append(AgreementForm(**kwargs))
+
+        kwargs['prefix'] = 'old_agreement'
+        self.old_agreement_form = self.OldAgreementForm(**kwargs)
+
+    def _add_error_required(self, field_name):
+        self.add_error(field_name, forms.ValidationError(
+            self.fields[field_name].error_messages['required'],
+            code='required',
+        ))
+
+    def clean(self):
+        # check required school and class
+        if 'participant_age_group' in self.cleaned_data:
+            if self.cleaned_data['participant_age_group'].require_school:
+                # require school
+                participant_school = self.cleaned_data.get('participant_school')
+                if participant_school:
+                    if participant_school == 'other':
+                        self.cleaned_data['participant_school'] = None
+                        # require other school
+                        if not self.cleaned_data.get('participant_school_other'):
+                            self._add_error_required('participant_school_other')
+                    else:
+                        self.cleaned_data['participant_school'] = School.objects.get(
+                            id=int(self.cleaned_data['participant_school']),
+                        )
+                else:
+                    self._add_error_required('participant_school')
+                # require school class
+                if not self.cleaned_data.get('participant_school_class'):
+                    self._add_error_required('participant_school_class')
+            else:
+                # delete school and class
+                self.cleaned_data['participant_school'] = None
+                self.cleaned_data['participant_school_other'] = ''
+                self.cleaned_data['participant_school_class'] = ''
+        return self.cleaned_data
+
+    @cached_property
+    def required_forms(self):
+        required_forms = [
+            super(RegistrationForm, self),
+            self.participant_select_form,
+            self.questions_form,
+            self.parent1_select_form,
+            self.parent2_select_form,
+        ] + self.agreement_forms
+        if not self.user.is_authenticated:
+            required_forms.append(self.email_form)
+        if self.instance.old_registration_agreement:
+            required_forms.append(self.old_agreement_form)
+        return required_forms
 
     def is_valid(self):
-        return (
-            self.participant_select_form.is_valid() and
-            self.questions_form.is_valid() and
-            self.parent1_select_form.is_valid() and
-            self.parent2_select_form.is_valid() and
-            self.agreement_form.is_valid() and
-            super(RegistrationForm, self).is_valid()
-        )
+        # validate all required forms
+        results = [form.is_valid() for form in self.required_forms]
+        return all(results)
+
+    @cached_property
+    def errors(self):
+        return {
+            (form.prefix + field_name if form.prefix else field_name): error
+            for form in self.required_forms for field_name, error in form.errors.items()
+        }
 
     def save(self, commit=True):
+        # set user
+        if self.user.is_authenticated():
+            self.instance.user = self.user
+        else:
+            email = self.email_form.cleaned_data['email'].lower()
+            user = User.objects.filter(email=email).first()
+            if not user:
+                username_base = email.split('@')[0]
+                username_max_length = User._meta.get_field('username').max_length
+                user = User(
+                    username=username_base[:username_max_length],
+                    email=email,
+                )
+                user.set_password(get_random_string())
+                while not user.pk:
+                    try:
+                        with transaction.atomic():
+                            user.save()
+                    except IntegrityError:
+                        user.username = '.'.join((
+                            username_base[:(username_max_length - 4)],
+                            get_random_string(3),
+                        ))
+            self.instance.user = user
+
+        # set price
         self.instance.price = (
             self.instance.subject_variant.price
             if self.instance.subject_variant
@@ -277,11 +389,16 @@ class RegistrationForm(FormMixin, forms.ModelForm):
                 setattr(parent, attr, getattr(self.instance.parent2, attr))
             parent.save()
 
-    class AgreementForm(FormMixin, forms.Form):
-        agreement = forms.BooleanField(
-            label       = _('Terms and Conditions agreement'),
-        )
+        for agreement_form in self.agreement_forms:
+            for option_id, checked in agreement_form.cleaned_data.items():
+                if checked:
+                    self.instance.agreement_options.add(agreement_form.options[option_id])
 
+    class OldAgreementForm(FormMixin, forms.Form):
+        agreement = forms.BooleanField(label = _('Terms and Conditions agreement'))
+
+    class EmailForm(FormMixin, forms.Form):
+        email = VerifiedEmailField(label=_('Your email'), fieldsetup_id='RegistrationEmailForm')
 
 
 class CourseRegistrationForm(RegistrationForm):
@@ -304,15 +421,27 @@ class RegistrationAdminForm(forms.ModelForm):
 
     def __init__(self, data=None, *args, **kwargs):
         super(RegistrationAdminForm, self).__init__(data, *args, **kwargs)
-        self.fields['subject_variant'].widget.choices.queryset = kwargs['instance'].subject.variants
-        if not kwargs['instance'].subject.variants.exists():
+        instance = kwargs.get('instance')
+        initial = kwargs.get('initial')
+        try:
+            subject = Subject.objects.get(id=int(data['subject']))
+        except (Subject.DoesNotExist, TypeError, KeyError, ValueError):
+            subject = instance.subject if instance else Subject.objects.get(id=initial['subject'])
+        self.fields['subject_variant'].widget.choices.queryset = subject.variants
+        if not subject.variants.exists():
             self.fields['subject_variant'].required = False
-        self.fields['participant_age_group'].widget.choices.queryset = kwargs['instance'].subject.age_groups
+        self.fields['participant_age_group'].widget.choices.queryset = subject.age_groups
 
         try:
-            age = get_age(get_birth_date(data['participant_birth_num']), kwargs['instance'].created.date())
+            age = get_age(
+                get_birth_date(data['participant_birth_num']),
+                instance.created.date() if instance else date.today(),
+            )
         except:
-            age = get_age(get_birth_date(kwargs['instance'].participant.birth_num), kwargs['instance'].created.date())
+            age = get_age(
+                get_birth_date(instance.participant.birth_num),
+                instance.created.date(),
+            ) if instance else 18
         if age < 18:
             self.fields['has_parent1'].required = True
         else:
@@ -322,8 +451,14 @@ class RegistrationAdminForm(forms.ModelForm):
         try:
             has_parent = ['has_parent1' in data, 'has_parent2' in data]
         except:
-            has_parent = [kwargs['instance'].has_parent1, kwargs['instance'].has_parent2]
+            has_parent = [instance.has_parent1, instance.has_parent2] if instance else [False, False]
         for n in range(2):
             if has_parent[n]:
                 for field in ['first_name', 'last_name', 'street', 'city', 'postal_code', 'phone', 'email']:
                     self.fields['parent{}_{}'.format(n + 1, field)].required = True
+
+        # choices for subject_variant
+        self.fields['agreement_options'].widget.choices = tuple(
+            (agreement.name, tuple((option.id, option.name) for option in agreement.all_options))
+            for agreement in subject.all_registration_agreements
+        )
