@@ -1,5 +1,3 @@
-from __future__ import unicode_literals
-
 from django.db import models
 from django.utils.encoding import python_2_unicode_compatible
 from django.utils.translation import ugettext_lazy as _
@@ -11,6 +9,7 @@ from ..utils import first_upper
 class AgeGroup(models.Model):
     name    = models.CharField(_('name'), max_length=50)
     order   = models.IntegerField(_('order'), blank=True, default=0)
+    require_school = models.BooleanField(_('require school'), default=True)
 
     class Meta:
         app_label           = 'leprikon'
