@@ -1,5 +1,3 @@
-from __future__ import unicode_literals
-
 import re
 
 from django import forms
@@ -33,6 +31,11 @@ class ColorField(models.CharField):
         return super(ColorField, self).formfield(**kwargs)
 
 
+class EmailField(models.EmailField):
+    def to_python(self, value):
+        v = super(EmailField, self).to_python(value)
+        return v and v.lower()
+
 
 class PriceField(models.DecimalField):
     def __init__(self, *args, **kwargs):
