@@ -1,35 +1,47 @@
-from __future__ import unicode_literals
-
 import colorsys
-from collections import OrderedDict
+import logging
+from collections import OrderedDict, namedtuple
+from email.mime.image import MIMEImage
 from io import BytesIO
+from itertools import chain
 from json import loads
 
+import qrcode
 import trml2pdf
+from bankreader.models import Transaction as BankreaderTransaction
 from cms.models.fields import PageField
 from django.core.exceptions import ValidationError
 from django.core.mail import EmailMultiAlternatives
 from django.core.urlresolvers import reverse_lazy as reverse
 from django.db import models
+from django.dispatch import receiver
 from django.template.loader import select_template
-from django.utils import timezone
+from django.utils import formats, timezone
 from django.utils.encoding import (
     force_text, python_2_unicode_compatible, smart_text,
 )
 from django.utils.functional import cached_property
 from django.utils.text import slugify
+from django.utils.timezone import now
 from django.utils.translation import ugettext_lazy as _
+from django_pays import payment_url as pays_payment_url
+from django_pays.models import Payment as PaysPayment
 from djangocms_text_ckeditor.fields import HTMLField
 from filer.fields.file import FilerFileField
 from filer.fields.image import FilerImageField
 from PyPDF2 import PdfFileReader, PdfFileWriter
 
 from ..conf import settings
-from ..utils import comma_separated, currency, get_birth_date
+from ..utils import (
+    comma_separated, currency, get_birth_date, localeconv, spayd,
+)
 from .agegroup import AgeGroup
+from .agreements import Agreement, AgreementOption
 from .citizenship import Citizenship
 from .department import Department
-from .fields import BirthNumberField, ColorField, PostalCodeField, PriceField
+from .fields import (
+    BirthNumberField, ColorField, EmailField, PostalCodeField, PriceField,
+)
 from .leprikonsite import LeprikonSite
 from .place import Place
 from .printsetup import PrintSetup
@@ -39,6 +51,8 @@ from .school import School
 from .schoolyear import SchoolYear
 from .utils import generate_variable_symbol
 
+logger = logging.getLogger(__name__)
+
 
 @python_2_unicode_compatible
 class SubjectType(models.Model):
@@ -63,7 +77,14 @@ class SubjectType(models.Model):
         Question, verbose_name=_('additional questions'), blank=True, related_name='+',
         help_text=_('Add additional questions to be asked in the registration form.'),
     )
-    agreement       = HTMLField(_('registration agreement'), blank=True, default='')
+    old_registration_agreement = HTMLField(
+        _('old registration agreement'), blank=True, default='',
+        help_text=_('This agreement will be removed in future version. Please, use registration agreements below.'),
+    )
+    registration_agreements = models.ManyToManyField(
+        Agreement, verbose_name=_('additional registration agreements'), blank=True, related_name='+',
+        help_text=_('Add additional legal agreements for the registration form.'),
+    )
     reg_print_setup = models.ForeignKey(PrintSetup, on_delete=models.SET_NULL, related_name='+',
                                         verbose_name=_('registration print setup'), blank=True, null=True)
     bill_print_setup = models.ForeignKey(PrintSetup, on_delete=models.SET_NULL, related_name='+',
@@ -185,13 +206,19 @@ class Subject(models.Model):
     questions   = models.ManyToManyField(Question, verbose_name=_('additional questions'),
                                          related_name='+', blank=True,
                                          help_text=_('Add additional questions to be asked in the registration form.'))
-    agreement       = HTMLField(_('registration agreement'), blank=True, default='')
+    old_registration_agreement = HTMLField(
+        _('old registration agreement'), blank=True, default='',
+        help_text=_('This agreement will be removed in future version. Please, use registration agreements below.'),
+    )
+    registration_agreements = models.ManyToManyField(
+        Agreement, verbose_name=_('additional registration agreements'), blank=True, related_name='+',
+        help_text=_('Add additional legal agreements for the registration form.'),
+    )
     reg_print_setup = models.ForeignKey(PrintSetup, on_delete=models.SET_NULL, related_name='+',
                                         verbose_name=_('registration print setup'), blank=True, null=True)
     bill_print_setup = models.ForeignKey(PrintSetup, on_delete=models.SET_NULL, related_name='+',
                                          verbose_name=_('payment print setup'), blank=True, null=True)
 
-
     class Meta:
         app_label           = 'leprikon'
         ordering            = ('code', 'name')
@@ -199,7 +226,21 @@ class Subject(models.Model):
         verbose_name_plural = _('subjects')
 
     def __str__(self):
-        return '{} {}'.format(self.school_year, self.name)
+        return '{} {}'.format(self.school_year, self.display_name)
+
+    @cached_property
+    def subject(self):
+        if self.subject_type.subject_type == self.subject_type.COURSE:
+            return self.course
+        else:
+            return self.event
+
+    @cached_property
+    def display_name(self):
+        if settings.LEPRIKON_SHOW_SUBJECT_CODE and self.code:
+            return '{} – {}'.format(self.code, self.name)
+        else:
+            return self.name
 
     @cached_property
     def all_variants(self):
@@ -241,7 +282,7 @@ class Subject(models.Model):
 
     @property
     def full(self):
-        return self.max_count and self.approved_registrations_count >= self.max_count
+        return self.max_count and self.active_registrations_count >= self.max_count
 
     def get_absolute_url(self):
         return reverse('leprikon:subject_detail', args=(self.subject_type.slug, self.id))
@@ -265,6 +306,10 @@ class Subject(models.Model):
     def active_registrations(self):
         return self.registrations.filter(canceled=None)
 
+    @cached_property
+    def active_registrations_count(self):
+        return self.active_registrations.count()
+
     @property
     def approved_registrations(self):
         return self.active_registrations.exclude(approved=None)
@@ -277,6 +322,13 @@ class Subject(models.Model):
     def all_approved_registrations(self):
         return list(self.approved_registrations.all())
 
+    @cached_property
+    def all_journal_periods(self):
+        if self.subject_type.subject_type == self.subject_type.COURSE:
+            return [JournalPeriod(self.course, period) for period in self.course.all_periods]
+        else:
+            return [JournalPeriod(self.event)]
+
     @property
     def unapproved_registrations(self):
         return self.active_registrations.filter(approved=None)
@@ -297,15 +349,87 @@ class Subject(models.Model):
     def all_inactive_registrations(self):
         return list(self.inactive_registrations.all())
 
-    def get_agreement(self):
+    def get_old_registration_agreement(self):
         return (
-            self.agreement or
-            self.subject_type.agreement or
-            LeprikonSite.objects.get_current().agreement or
-            _('I confirm that I have read, understood and agree with the Terms and Conditions. '
-              'The terms and conditions are available on the web.')
+            self.old_registration_agreement or
+            self.subject_type.old_registration_agreement or
+            LeprikonSite.objects.get_current().old_registration_agreement
         )
 
+    @cached_property
+    def all_registration_agreements(self):
+        return sorted(
+            chain(
+                self.registration_agreements.all(),
+                self.subject_type.registration_agreements.all(),
+                LeprikonSite.objects.get_current().registration_agreements.all(),
+            ),
+            key=lambda agreement: agreement.order
+        )
+
+
+class JournalPeriod:
+    def __init__(self, subject, period=None):
+        self.subject = subject
+        self.period = period
+
+    @property
+    def all_journal_entries(self):
+        qs = self.subject.journal_entries.all()
+        if self.period:
+            qs = qs.filter(date__gte=self.period.start, date__lte=self.period.end)
+        return list(qs)
+
+    @cached_property
+    def all_registrations(self):
+        if self.period:  # course
+            qs = self.subject.registrations_history_registrations.filter(created__lt=self.period.end)
+        else:  # event
+            qs = self.subject.registrations.all()
+        return list(qs)
+
+    @cached_property
+    def all_alternates(self):
+        alternates = set()
+        for entry in self.all_journal_entries:
+            for alternate in entry.all_alternates:
+                alternates.add(alternate)
+        return list(alternates)
+
+    PresenceRecord = namedtuple('PresenceRecord', ('name', 'presences'))
+
+    def get_participant_presences(self):
+        return [
+            self.PresenceRecord(
+                reg.participant,
+                [
+                    reg.id in entry.all_registrations_idset
+                    for entry in self.all_journal_entries
+                ]
+            ) for reg in self.all_registrations
+        ]
+
+    def get_leader_presences(self):
+        return [
+            self.PresenceRecord(
+                leader,
+                [
+                    entry.all_leader_entries_by_leader.get(leader, None)
+                    for entry in self.all_journal_entries
+                ]
+            ) for leader in self.subject.all_leaders
+        ]
+
+    def get_alternate_presences(self):
+        return [
+            self.PresenceRecord(
+                alternate,
+                [
+                    entry.all_leader_entries_by_leader.get(alternate, None)
+                    for entry in self.all_journal_entries
+                ]
+            ) for alternate in self.all_alternates
+        ]
 
 
 @python_2_unicode_compatible
@@ -441,6 +565,7 @@ class SubjectRegistration(PdfExportMixin, models.Model):
     payment_requested = models.DateTimeField(_('payment request time'), editable=False, null=True)
     canceled        = models.DateTimeField(_('time of cancellation'), editable=False, null=True)
     cancel_request  = models.BooleanField(_('cancel request'), default=False)
+    note            = models.CharField(_('note'), max_length=300, blank=True, default='')
 
     MALE = 'm'
     FEMALE = 'f'
@@ -457,7 +582,7 @@ class SubjectRegistration(PdfExportMixin, models.Model):
     participant_citizenship = models.ForeignKey(Citizenship, verbose_name=_('citizenship'),
                                                 related_name='+', on_delete=models.PROTECT)
     participant_phone       = models.CharField(_('phone'),        max_length=30, blank=True, default='')
-    participant_email       = models.EmailField(_('email address'),              blank=True, default='')
+    participant_email       = EmailField(_('email address'),              blank=True, default='')
 
     participant_school          = models.ForeignKey(School, verbose_name=_('school'), blank=True, null=True,
                                                     related_name='+', on_delete=models.PROTECT)
@@ -472,7 +597,7 @@ class SubjectRegistration(PdfExportMixin, models.Model):
     parent1_city        = models.CharField(_('city'),         max_length=150, blank=True, null=True)
     parent1_postal_code = PostalCodeField(_('postal code'),                   blank=True, null=True)
     parent1_phone       = models.CharField(_('phone'),        max_length=30,  blank=True, null=True)
-    parent1_email       = models.EmailField(_('email address'),               blank=True, null=True)
+    parent1_email       = EmailField(_('email address'),               blank=True, null=True)
 
     has_parent2         = models.BooleanField(_('second parent'), default=False)
     parent2_first_name  = models.CharField(_('first name'),   max_length=30,  blank=True, null=True)
@@ -481,7 +606,9 @@ class SubjectRegistration(PdfExportMixin, models.Model):
     parent2_city        = models.CharField(_('city'),         max_length=150, blank=True, null=True)
     parent2_postal_code = PostalCodeField(_('postal code'),                   blank=True, null=True)
     parent2_phone       = models.CharField(_('phone'),        max_length=30,  blank=True, null=True)
-    parent2_email       = models.EmailField(_('email address'),               blank=True, null=True)
+    parent2_email       = EmailField(_('email address'),               blank=True, null=True)
+
+    agreement_options   = models.ManyToManyField(AgreementOption, blank=True)
 
     variable_symbol     = models.BigIntegerField(_('variable symbol'), db_index=True, editable=False, null=True)
 
@@ -499,7 +626,7 @@ class SubjectRegistration(PdfExportMixin, models.Model):
     def validate_unique(self, exclude=None):
         super(SubjectRegistration, self).validate_unique(None)
         qs = SubjectRegistration.objects.filter(
-            subject=self.subject,
+            subject_id=self.subject_id,
             participant_birth_num=self.participant_birth_num,
             canceled=None)
         if self.pk:
@@ -552,6 +679,10 @@ class SubjectRegistration(PdfExportMixin, models.Model):
             return None
     parent2.short_description = _('second parent')
 
+    @cached_property
+    def all_agreement_options(self):
+        return list(self.agreement_options.all())
+
     @cached_property
     def all_attachments(self):
         return self.subject.all_attachments + self.subject.subject_type.all_attachments
@@ -574,15 +705,41 @@ class SubjectRegistration(PdfExportMixin, models.Model):
     def all_payments(self):
         return list(self.payments.all())
 
+    @cached_property
+    def current_receivable(self):
+        return self.subjectregistration.current_receivable
+
+    @cached_property
+    def spayd(self):
+        leprikon_site = LeprikonSite.objects.get_current()
+        return spayd(
+            ('ACC', ('%s+%s' % (leprikon_site.iban, leprikon_site.bic)) if leprikon_site.bic else leprikon_site.iban),
+            ('AM', self.current_receivable),
+            ('CC', localeconv['int_curr_symbol'].strip()),
+            ('MSG', '%s, %s' % (self.subject.name[:29], self.participant.full_name[:29])),
+            ('RN', slugify(leprikon_site.company_name).replace('*', '')[:35]),
+            ('X-VS', self.variable_symbol),
+        )
+
+    @cached_property
+    def payment_url(self):
+        leprikon_site = LeprikonSite.objects.get_current()
+        return pays_payment_url(
+            gateway=leprikon_site.payment_gateway,
+            order_id=self.variable_symbol,
+            amount=self.current_receivable,
+            email=self.user.email,
+        )
+
     def get_discounts(self, d):
         if d:
-            return list(filter(lambda p: p.created.date() <= d, self.all_discounts))
+            return [p for p in self.all_discounts if p.accounted.date() <= d]
         else:
             return self.all_discounts
 
     def get_payments(self, d):
         if d:
-            return list(filter(lambda p: p.created.date() <= d, self.all_payments))
+            return [p for p in self.all_payments if p.accounted.date() <= d]
         else:
             return self.all_payments
 
@@ -592,25 +749,32 @@ class SubjectRegistration(PdfExportMixin, models.Model):
     def get_paid(self, d=None):
         return sum(p.amount for p in self.get_payments(d))
 
-    def get_current_receivable(self):
-        return self.subjectregistration.get_current_receivable()
+    def get_qr_code(self):
+        output = BytesIO()
+        qrcode.make(self.spayd, border=1).save(output)
+        output.seek(0)
+        return output.read()
 
     def approve(self):
         if self.approved is None:
             self.approved = timezone.now()
             self.save()
+        try:
+            self.send_registration_approval()
+        except Exception as e:
+            logger.exception(e)
+        if self.payment_requested is None:
+            self.request_payment()
 
     def request_payment(self):
         if self.payment_requested is None:
             self.payment_requested = timezone.now()
             self.save()
+        if self.current_receivable:
             try:
                 self.send_payment_request()
-            except:
-                import traceback
-                from raven.contrib.django.models import client
-                traceback.print_exc()
-                client.captureException()
+            except Exception as e:
+                logger.exception(e)
 
     def cancel(self):
         if self.canceled is None:
@@ -656,8 +820,8 @@ class SubjectRegistration(PdfExportMixin, models.Model):
             super(SubjectRegistration, self).save(*args, **kwargs)
 
     @cached_property
-    def agreement(self):
-        return self.subject.get_agreement()
+    def old_registration_agreement(self):
+        return self.subject.get_old_registration_agreement()
 
     pdf_export = 'registration'
 
@@ -672,14 +836,23 @@ class SubjectRegistration(PdfExportMixin, models.Model):
 
     @cached_property
     def mail_subject(self):
-        return _('Registration for {subject_type} accepted').format(
-            subject_type=self.subject.subject_type.name_akuzativ
+        return _('Registration for {subject_type} {subject}').format(
+            subject_type=self.subject.subject_type.name_akuzativ,
+            subject=self.subject.name,
         )
 
     @cached_property
     def payment_request_subject(self):
-        return _('Registration for {subject_type} - payment information').format(
-            subject_type=self.subject.subject_type.name_akuzativ
+        return _('Registration for {subject_type} {subject} - payment request').format(
+            subject_type=self.subject.subject_type.name_akuzativ,
+            subject=self.subject.name,
+        )
+
+    @cached_property
+    def registration_approval_subject(self):
+        return _('Registration for {subject_type} {subject} - approved').format(
+            subject_type=self.subject.subject_type.name_akuzativ,
+            subject=self.subject.name,
         )
 
     def send_payment_request(self):
@@ -693,9 +866,10 @@ class SubjectRegistration(PdfExportMixin, models.Model):
             'leprikon/payment-request-mail/{}.html'.format(self.subject.subject_type.subject_type),
             'leprikon/payment-request-mail/subject.html',
         ])
+        site = LeprikonSite.objects.get_current()
         context = {
             'object': self,
-            'site': LeprikonSite.objects.get_current(),
+            'site': site,
         }
         content_txt = template_txt.render(context)
         content_html = template_html.render(context)
@@ -707,6 +881,38 @@ class SubjectRegistration(PdfExportMixin, models.Model):
             headers     = {'X-Mailer': 'Leprikon (http://leprikon.cz/)'},
         )
         msg.attach_alternative(content_html, 'text/html')
+        if site.bank_account:
+            qr_code = MIMEImage(self.get_qr_code())
+            qr_code.add_header('Content-ID', '<qr_code>')
+            msg.attach(qr_code)
+        msg.send()
+
+    def send_registration_approval(self):
+        template_txt = select_template([
+            'leprikon/registration-approved-mail/{}.txt'.format(self.subject.subject_type.slug),
+            'leprikon/registration-approved-mail/{}.txt'.format(self.subject.subject_type.subject_type),
+            'leprikon/registration-approved-mail/subject.txt',
+        ])
+        template_html = select_template([
+            'leprikon/registration-approved-mail/{}.html'.format(self.subject.subject_type.slug),
+            'leprikon/registration-approved-mail/{}.html'.format(self.subject.subject_type.subject_type),
+            'leprikon/registration-approved-mail/subject.html',
+        ])
+        site = LeprikonSite.objects.get_current()
+        context = {
+            'object': self,
+            'site': site,
+        }
+        content_txt = template_txt.render(context)
+        content_html = template_html.render(context)
+        msg = EmailMultiAlternatives(
+            subject = self.registration_approval_subject,
+            body        = content_txt,
+            from_email  = settings.SERVER_EMAIL,
+            to          = self.all_recipients,
+            headers     = {'X-Mailer': 'Leprikon (http://leprikon.cz/)'},
+        )
+        msg.attach_alternative(content_html, 'text/html')
         msg.send()
 
     @python_2_unicode_compatible
@@ -753,10 +959,31 @@ class SubjectRegistration(PdfExportMixin, models.Model):
             return get_birth_date(self.birth_num)
 
 
+class TransactionMixin(object):
+
+    def save(self, *args, **kwargs):
+        self.clean()
+        super(TransactionMixin, self).save(*args, **kwargs)
+
+    def clean(self):
+        errors = {}
+        if self.amount == 0:
+            errors['amount'] = [_('Amount can not be zero.')]
+        if self.accounted:
+            max_closure_date = LeprikonSite.objects.get_current().max_closure_date
+            if max_closure_date and self.accounted.date() <= max_closure_date:
+                errors['accounted'] = [
+                    _('Date must be after the last account closure ({}).').format(
+                        formats.date_format(max_closure_date)
+                    )
+                ]
+        if errors:
+            raise ValidationError(errors)
+
 
 @python_2_unicode_compatible
-class SubjectDiscount(models.Model):
-    created     = models.DateTimeField(_('time of discount'), editable=False, auto_now_add=True)
+class SubjectDiscount(TransactionMixin, models.Model):
+    accounted   = models.DateTimeField(_('accounted time'), default=now)
     amount      = PriceField(_('discount'), default=0)
     explanation = models.CharField(_('discount explanation'), max_length=250, blank=True, default='')
 
@@ -765,19 +992,14 @@ class SubjectDiscount(models.Model):
         app_label           = 'leprikon'
         verbose_name        = _('discount')
         verbose_name_plural = _('discounts')
-        ordering            = ('created',)
+        ordering            = ('accounted',)
 
     def __str__(self):
         return currency(self.amount)
 
-    def save(self, *args, **kwargs):
-        self.registration.approve()
-        super(SubjectDiscount, self).save(*args, **kwargs)
-
-
 
 @python_2_unicode_compatible
-class SubjectPayment(PdfExportMixin, models.Model):
+class SubjectPayment(PdfExportMixin, TransactionMixin, models.Model):
     PAYMENT_CASH = 'PAYMENT_CASH'
     PAYMENT_BANK = 'PAYMENT_BANK'
     PAYMENT_ONLINE = 'PAYMENT_ONLINE'
@@ -794,25 +1016,31 @@ class SubjectPayment(PdfExportMixin, models.Model):
         (RETURN_BANK, _('return - bank')),
         (RETURN_TRANSFER, _('return - transfer to payment')),
     ])
+    payments = frozenset({PAYMENT_CASH, PAYMENT_BANK, PAYMENT_ONLINE, PAYMENT_TRANSFER})
+    returns = frozenset({RETURN_CASH, RETURN_BANK, RETURN_TRANSFER})
+
     registration    = models.ForeignKey(SubjectRegistration, verbose_name=_('registration'),
                                         related_name='payments', on_delete=models.PROTECT)
-    created         = models.DateTimeField(_('payment time'), editable=False, auto_now_add=True)
+    accounted       = models.DateTimeField(_('accounted time'), default=now)
     payment_type    = models.CharField(_('payment type'), max_length=30, choices=payment_type_labels.items())
     amount          = PriceField(_('amount'), help_text=_('positive value for payment, negative value for return'))
     note            = models.CharField(_('note'), max_length=300, blank=True, default='')
-    related_payment = models.ForeignKey('self', verbose_name=_('related payment'), blank=True, null=True,
-                                        related_name='related_payments', on_delete=models.PROTECT)
+    related_payment = models.OneToOneField('self', verbose_name=_('related payment'), blank=True, null=True,
+                                           limit_choices_to={'payment_type__in': (PAYMENT_TRANSFER, RETURN_TRANSFER)},
+                                           related_name='related_payments', on_delete=models.PROTECT)
+    bankreader_transaction = models.OneToOneField(BankreaderTransaction, verbose_name=_('bank account transaction'),
+                                                  blank=True, null=True, on_delete=models.PROTECT)
+    pays_payment    = models.OneToOneField(PaysPayment, editable=False, verbose_name=_('online payment'),
+                                           limit_choices_to={'status': PaysPayment.REALIZED},
+                                           blank=True, null=True, on_delete=models.PROTECT)
     received_by     = models.ForeignKey(settings.AUTH_USER_MODEL, null=True, verbose_name=_('received by'),
                                         related_name='+', on_delete=models.PROTECT)
 
-    payments = frozenset({PAYMENT_CASH, PAYMENT_BANK, PAYMENT_ONLINE, PAYMENT_TRANSFER})
-    returns = frozenset({RETURN_CASH, RETURN_BANK, RETURN_TRANSFER})
-
     class Meta:
         app_label           = 'leprikon'
         verbose_name        = _('payment')
         verbose_name_plural = _('payments')
-        ordering            = ('created',)
+        ordering            = ('accounted',)
 
     def __str__(self):
         return '{registration}, {payment_type} {amount}'.format(
@@ -847,30 +1075,97 @@ class SubjectPayment(PdfExportMixin, models.Model):
 
     @cached_property
     def mail_subject(self):
-        return _('Registration for {subject_type} accepted').format(
-            subject_type=self.subject.subject_type.name_akuzativ
+        return _('Registration for {subject_type} {subject} - payment confirmation').format(
+            subject_type=self.subject.subject_type.name_akuzativ,
+            subject=self.subject.name,
         )
 
-    def validate(self):
-        if self.amount == 0:
-            raise ValidationError({
-                'amount': [_('Amount can not be zero.')]
-            })
-        if self.payment_type:
-            if self.amount > 0 and self.payment_type not in self.payments:
-                raise ValidationError({
-                    'payment_type': [_('Select one of the payments or use negative amount.')],
-                })
-            if self.amount < 0 and self.payment_type not in self.returns:
-                raise ValidationError({
-                    'payment_type': [_('Select one of the returns or use positive amount.')],
-                })
+    def clean(self):
+        errors = {}
+        try:
+            super(SubjectPayment, self).clean()
+        except ValidationError as e:
+            errors = e.update_error_dict(errors)
 
-    def clean_fields(self, exclude=None):
-        super(SubjectPayment, self).clean_fields(exclude)
-        self.validate()
+        # ensure at most one relation
+        if [self.related_payment, self.bankreader_transaction, self.pays_payment].count(None) < 2:
+            message = _('Payment can only have one relation.')
+            if self.related_payment:
+                errors['related_payment'] = message
+            if self.bankreader_transaction:
+                errors['bankreader_transaction'] = message
+        else:
+            if self.related_payment:
+                valid_payment_type = (
+                    SubjectPayment.PAYMENT_TRANSFER
+                    if self.related_payment.payment_type == SubjectPayment.RETURN_TRANSFER
+                    else SubjectPayment.RETURN_TRANSFER
+                )
+                valid_amount = -self.related_payment.amount
+            if self.bankreader_transaction:
+                valid_payment_type = (
+                    SubjectPayment.PAYMENT_BANK
+                    if self.bankreader_transaction.amount > 0
+                    else SubjectPayment.RETURN_BANK
+                )
+                valid_amount = self.bankreader_transaction.amount
+            if self.related_payment or self.bankreader_transaction:
+                if self.payment_type != valid_payment_type:
+                    errors.setdefault('related_payment', []).append(
+                        _('Payment type must be {valid_payment_type}.').format(
+                            valid_payment_type=self.payment_type_labels[valid_payment_type],
+                        )
+                    )
+                if self.amount != valid_amount:
+                    errors.setdefault('amount', []).append(
+                        _('Amount must be {valid_amount}.').format(valid_amount=valid_amount)
+                    )
+        if errors:
+            raise ValidationError(errors)
 
-    def save(self, *args, **kwargs):
-        self.validate()
-        self.registration.approve()
-        super(SubjectPayment, self).save(*args, **kwargs)
+
+@receiver(models.signals.post_save, sender=PaysPayment)
+def payment_create_subject_payment(instance, **kwargs):
+    payment = instance
+    # check realized payment
+    if payment.status != PaysPayment.REALIZED:
+        return
+    # check registration
+    try:
+        registration = SubjectRegistration.objects.get(variable_symbol=int(payment.order_id))
+    except (ValueError, SubjectRegistration.DoesNotExist):
+        return
+    # create payment
+    SubjectPayment.objects.create(
+        registration=registration,
+        accounted=payment.created,
+        payment_type=SubjectPayment.PAYMENT_ONLINE,
+        amount=payment.amount / payment.base_units,
+        note=_('received online payment'),
+        pays_payment=payment,
+    )
+
+
+@receiver(models.signals.post_save, sender=BankreaderTransaction)
+def transaction_create_subject_payment(instance, **kwargs):
+    transaction = instance
+    # check variable symbol
+    if not transaction.variable_symbol:
+        return
+    # check closure date (use closure date from cached leprikon site)
+    max_closure_date = LeprikonSite.objects.get_current().max_closure_date
+    if max_closure_date and transaction.accounted_date <= max_closure_date:
+        return
+    # check registration
+    registration = SubjectRegistration.objects.filter(variable_symbol=transaction.variable_symbol).first()
+    if not registration:
+        return
+    # create payment
+    SubjectPayment.objects.create(
+        registration=registration,
+        accounted=transaction.accounted_date,
+        payment_type=SubjectPayment.PAYMENT_BANK if transaction.amount >= 0 else SubjectPayment.RETURN_BANK,
+        amount=transaction.amount,
+        note=_('imported from account statement'),
+        bankreader_transaction=transaction,
+    )
