@@ -1,9 +1,10 @@
-from __future__ import unicode_literals
-
 import locale
 import os
 import string
+import unicodedata
+import zlib
 from datetime import date
+from urllib.parse import urlencode
 
 from django.core.exceptions import ObjectDoesNotExist
 from django.core.urlresolvers import reverse_lazy as reverse
@@ -13,13 +14,8 @@ from django.utils.translation import ugettext_lazy as _
 
 from .conf import settings
 
-try:
-    from urllib import urlencode
-except ImportError:
-    from urllib.parse import urlencode
 
-
-def _get_localeconv(languages):
+def _get_localeconv():
     """
     This function loads localeconv during module load.
     It is necessary, because using locale.setlocale later may be dangerous
@@ -33,7 +29,7 @@ def _get_localeconv(languages):
     return lc
 
 
-localeconv = _get_localeconv(settings.LANGUAGE_CODE)
+localeconv = _get_localeconv()
 
 
 # This function is inspired by python's standard locale.currency().
@@ -178,8 +174,16 @@ def merge_objects(source, target, attributes=None, exclude=[]):
 @transaction.atomic
 def merge_users(source, target):
     target = merge_objects(source, target, ('first_name', 'last_name', 'email'))
-    target.date_joined = min(source.date_joined, target.date_joined)
-    target.last_login = max(source.last_login, target.last_login)
+    target.date_joined = (
+        min(source.date_joined, target.date_joined)
+        if source.date_joined and target.date_joined
+        else source.date_joined or target.date_joined
+    )
+    target.last_login = (
+        max(source.last_login, target.last_login)
+        if source.last_login and target.last_login
+        else source.last_login or target.last_login
+    )
 
     try:
         leader = source.leprikon_leader
@@ -223,3 +227,12 @@ def merge_users(source, target):
         pass
 
     source.delete()
+
+
+def spayd(*items):
+    s = 'SPD*1.0*' + '*'.join(
+        '%s:%s' % (k, unicodedata.normalize('NFKD', str(v).replace('*', '')).encode('ascii', 'ignore').upper().decode())
+        for k, v in sorted(items)
+    )
+    s += '*CRC32:%x' % zlib.crc32(s.encode())
+    return s.upper()
