@@ -1,5 +1,3 @@
-from __future__ import unicode_literals
-
 from collections import namedtuple
 
 from django.core.urlresolvers import reverse_lazy as reverse
@@ -8,7 +6,6 @@ from django.template.response import TemplateResponse
 from django.utils.functional import cached_property
 from django.utils.translation import ugettext_lazy as _
 
-from . import ReportBaseView
 from ...forms.reports.courses import (
     CoursePaymentsForm, CoursePaymentsStatusForm, CourseStatsForm,
 )
@@ -17,6 +14,7 @@ from ...models.citizenship import Citizenship
 from ...models.courses import Course, CourseRegistration
 from ...models.roles import Participant
 from ...models.subjects import SubjectPayment, SubjectType
+from . import ReportBaseView
 
 
 class ReportCoursePaymentsView(ReportBaseView):
@@ -31,8 +29,8 @@ class ReportCoursePaymentsView(ReportBaseView):
         context['form'] = form
         context['payments'] = SubjectPayment.objects.filter(
             registration__subject__subject_type__subject_type=SubjectType.COURSE,
-            created__gte=context['date_start'],
-            created__lte=context['date_end'],
+            accounted__gte=context['date_start'],
+            accounted__lte=context['date_end'],
         )
         context['sum'] = context['payments'].aggregate(Sum('amount'))['amount__sum']
         return TemplateResponse(self.request, self.template_name, self.get_context_data(**context))
