@@ -1,5 +1,3 @@
-from __future__ import unicode_literals
-
 from django.core.urlresolvers import reverse_lazy as reverse
 from django.http import HttpResponse
 from django.shortcuts import get_object_or_404, redirect
@@ -11,6 +9,7 @@ from ..forms.subjects import (
 )
 from ..models.courses import Course
 from ..models.events import Event
+from ..models.leprikonsite import LeprikonSite
 from ..models.subjects import (
     Subject, SubjectPayment, SubjectRegistration, SubjectType,
 )
@@ -112,6 +111,10 @@ class SubjectRegistrationsView(SubjectTypeMixin, DetailView):
             qs = qs.filter(leaders=self.request.leader)
         return qs
 
+    def get_context_data(self, **kwargs):
+        context = super(SubjectRegistrationsView, self).get_context_data(**kwargs)
+        context['site'] = LeprikonSite.objects.get_current()
+        return context
 
 
 class SubjectUpdateView(SubjectTypeMixin, UpdateView):
@@ -140,7 +143,7 @@ class SubjectUpdateView(SubjectTypeMixin, UpdateView):
 class SubjectRegistrationFormView(CreateView):
     back_url        = reverse('leprikon:registration_list')
     submit_label    = _('Submit registration')
-    message         = _('The registration has been accepted.')
+    message         = _('The registration has been saved. We will inform you about its further processing.')
     _models = {
         SubjectType.COURSE: Course,
         SubjectType.EVENT:  Event,
@@ -241,7 +244,7 @@ class UserPaymentMixin(object):
     def get_queryset(self):
         return super(UserPaymentMixin, self).get_queryset().filter(
             registration__user=self.request.user
-        ).order_by('-created')
+        ).order_by('-accounted')
 
 
 class SubjectPaymentsListView(UserPaymentMixin, ListView):
